######################################################################################################
##
##  Reference implementation of the KSN format for GNU R
##
##  Copyright (c) 2009-2011 Stanisław A. Raczyński,
##                          Sagayama Laboratory,
##                          The University of Tokyo
##
######################################################################################################

.pitchNamesSharp = c("C", "+C", "D", "+D", "E", "F", "+F", "G", "+G", "A", "+A", "B")
.pitchNamesFlat  = c("C", "-D", "D", "-E", "E", "F", "-G", "G", "-A", "A", "-B", "B")

.majorDegrees         = which(c(1,0,1,0,1,1,0,1,0,1,0,1) == 1) - 1  # major scale intervals
.minorNaturalDegrees  = which(c(1,0,1,1,0,1,0,1,1,0,1,0) == 1) - 1  # natural minor scale intervals
.minorHarmonicDegrees = which(c(1,0,1,1,0,1,0,1,1,0,0,1) == 1) - 1  # harmonic minor scale intervals
.minorMelodicDegrees  = which(c(1,0,1,1,0,1,0,1,0,1,0,1) == 1) - 1  # melodic minor scale intervals

.keySignatures = rbind(c(11, 6, 1, 8, 3, 10, 5, 0, 7, 2, 9, 4, 11, 6, 1),  # major
                       c(8, 3, 10, 5, 0, 7, 2, 9, 4, 11, 6, 1, 8, 3, 10))  # minor

# This is used to convert enharmonic spellings, like '##b' to names that can be used in e.g. '.pitchNamesSharp' table
.enharmonicSpellings        <- c("-B",  "a", "+c", "-E",  "f",  "E",  "e",  "+f", "-A", "B",  "G",   "-A")
names(.enharmonicSpellings) <- c("+A", "--b", "-d", "+D", "+e", "-F", "-f", "-g", "+G", "-c", "--A", "--B")

# Names of degrees of a chord
.degreeNames = c("Root", "Second", "Third", "Fourth", "Fifth", "Sixth", "Seventh", "Eighth", "Ninth", "Tenth", "Eleventh", "Twelfth", "Thirteenth", "Fourteenth", "Fifteenth")

.keysWithSharps = c("C", "a", "G", "e", "D", "b", "A", "+f", "E", "+c", "B", "+g", "+F", "+d", "+C", "+a")
.keysWithFlats = c("F", "d", "-B", "g", "-E", "c", "-A", "f", "-D", "-b", "-G", "-e", "-C", "-a")

.keyModifiers <- matrix(0, 30, 7)
rownames(.keyModifiers) <- c("C", "D", "E", "F", "G", "A", "B", "c", "d", "e", "f", "g", "a", "b", "+C", "+d", "+F", "+f", "+g", "+a", "+c", "-C", "-D", "-E", "-e", "-G", "-A", "-a", "-B", "-b")
.keyModifiers[1,]  <-  1*c(0, 0, 0, 0, 0, 0, 0)  # C-major
.keyModifiers[2,]  <-  1*c(0, 0, 1, 0, 0, 0, 1)  # D-major
.keyModifiers[3,]  <-  1*c(0, 1, 1, 0, 0, 1, 1)  # E-major
.keyModifiers[4,]  <- -1*c(0, 0, 1, 0, 0, 0, 0)  # F-major
.keyModifiers[5,]  <-  1*c(0, 0, 0, 0, 0, 0, 1)  # G-major
.keyModifiers[6,]  <-  1*c(0, 0, 1, 0, 0, 1, 1)  # A-major
.keyModifiers[7,]  <-  1*c(0, 1, 1, 0, 1, 1, 1)  # B-major
.keyModifiers[8,]  <- -1*c(0, 0, 1, 0, 0, 1, 1)  # c-minor
.keyModifiers[9,]  <- -1*c(0, 0, 0, 0, 0, 1, 0)  # d-minor
.keyModifiers[10,] <-  1*c(0, 1, 0, 0, 0, 0, 0)  # e-minor
.keyModifiers[11,] <- -1*c(0, 0, 1, 1, 0, 1, 1)  # f-minor
.keyModifiers[12,] <- -1*c(0, 0, 1, 0, 0, 1, 0)  # g-minor
.keyModifiers[13,] <- -1*c(0, 0, 0, 0, 0, 0, 0)  # a-minor
.keyModifiers[14,] <-  1*c(0, 1, 0, 0, 1, 0, 0)  # b-minor
.keyModifiers[15,] <-  1*c(1, 1, 1, 1, 1, 1, 1)  # C#-major
.keyModifiers[16,] <-  1*c(1, 1, 1, 1, 1, 0, 1)  # d#-minor
.keyModifiers[17,] <-  1*c(1, 1, 1, 0, 1, 1, 1)  # F#-major
.keyModifiers[18,] <-  1*c(1, 1, 0, 0, 1, 0, 0)  # f#-minor
.keyModifiers[19,] <-  1*c(1, 1, 0, 1, 1, 0, 1)  # g#-minor
.keyModifiers[20,] <-  1*c(1, 1, 1, 1, 1, 1, 1)  # a#-minor
.keyModifiers[21,] <-  1*c(1, 1, 0, 1, 1, 0, 0)  # c#-minor
.keyModifiers[22,] <- -1*c(1, 1, 1, 1, 1, 1, 1)  # Cb-major
.keyModifiers[23,] <- -1*c(1, 1, 0, 1, 1, 1, 0)  # Db-major
.keyModifiers[24,] <- -1*c(1, 0, 0, 1, 1, 0, 0)  # Eb-major
.keyModifiers[25,] <- -1*c(1, 0, 1, 1, 1, 1, 1)  # eb-minor
.keyModifiers[26,] <- -1*c(1, 1, 1, 1, 1, 1, 0)  # Gb-major
.keyModifiers[27,] <- -1*c(1, 1, 0, 1, 1, 0, 0)  # Ab-major
.keyModifiers[28,] <- -1*c(1, 1, 1, 1, 1, 1, 1)  # ab-minor
.keyModifiers[29,] <- -1*c(1, 0, 0, 1, 0, 0, 0)  # Bb-major
.keyModifiers[30,] <- -1*c(1, 0, 1, 1, 0, 1, 1)  # bb-minor

                     # 1  2  3  4  5  6  7   8   9   10  11  12  13  14  15
.standardIntervals = c(0, 2, 4, 5, 7, 9, 11, 12, 14, 16, 18, 19, 21, 23, 24)
                     # The above are all major or perfect intervals.
                     # Minor and diminished intervals are notated with
                     # a flat, and augmented intervals with a sharp in Harte notation

# Names used in Harte's notation
.pitchNamesSharp.Harte = c("C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B")
.pitchNamesFlat.Harte  = c("C", "Db", "D", "Eb", "E", "F", "Gb", "G", "Ab", "A", "Bb", "B")
.modifiers.Harte = c("bb", "b", "", "#", "##")


